// Copyright (c) Mito
import React, { useEffect, useState } from 'react';
import DefaultTaskpane from '../DefaultTaskpane';
// Import 
import '../../../../css/import-taskpane.css';
import LargeSelect from '../../elements/LargeSelect';
import XIcon from '../../icons/XIcon';
import ImportByPathCard from './ImportByPathCard';
/*
    The FileImportType is used to tell the UI which import type to display:
    - CURRENT_FOLDER => display select import
    - BY_PATH => input field for paths.
*/
export var FileImportType;
(function (FileImportType) {
    FileImportType["CURRENT_FOLDER"] = "current_folder";
    FileImportType["BY_PATH"] = "by_path";
})(FileImportType || (FileImportType = {}));
/*
    Provides a live updating import modal
    that allows users to import data
*/
function ImportTaskpane(props) {
    // Save the step id for overwriting
    const [stepID, setStepID] = useState('');
    // The datafiles are the files loaded from the backend
    const [dataFiles, setDataFiles] = useState([]);
    const [hasLoadedDataFiles, setHasLoadedDataFiles] = useState(false);
    // Which files are actually selected by the user
    const [selectedFiles, setSelectedFiles] = useState([]);
    /*
        Sends a message to the backend, importing files (and overwriting
        the step if it already exists).
    */
    async function sendImportMessage(files) {
        // Collect all of the file paths to submit 
        let filePaths = files.map(file => file.path);
        // Remove any empty file paths
        filePaths = filePaths.filter(filePath => filePath !== '');
        // Send message to backend to process import
        const newStepID = await props.mitoAPI.sendSimpleImportMessage(filePaths, stepID);
        setStepID(newStepID);
    }
    /*
        Loads the data files from the backend.
    */
    async function getDataFiles() {
        // Loads the data files
        const receivedDataFiles = await props.mitoAPI.getDataFiles();
        setDataFiles(receivedDataFiles);
        // Save that we have loaded the files
        setHasLoadedDataFiles(true);
    }
    // Updates a specific imported file to a specific import
    const updateImportedFile = async (index, newImportPath, importType) => {
        const newImports = [...selectedFiles];
        newImports[index] = {
            path: newImportPath,
            importType: importType
        };
        setSelectedFiles(newImports);
        // Set a message to actually update this
        await sendImportMessage(newImports);
    };
    // Adds a new file to import by current folder select
    const addNewCurrentFolderFileImport = async () => {
        const newImports = [...selectedFiles];
        // We take the first file that has not been imported, or if all files have been
        // imported, than just the first file
        let newImport = dataFiles[0];
        const selectedFilePaths = selectedFiles.map(fileObj => fileObj.path);
        const unimportedFiles = dataFiles.filter(fileName => !selectedFilePaths.includes(fileName));
        if (unimportedFiles.length > 0) {
            newImport = unimportedFiles[0];
        }
        newImports.push({
            path: newImport,
            importType: FileImportType.CURRENT_FOLDER
        });
        setSelectedFiles(newImports);
        // Send a message to actually update this
        await sendImportMessage(newImports);
    };
    // Adds a new file to import by path.
    // We ue it to create the UI input field
    const addNewByPathFileImport = () => {
        const newImports = [...selectedFiles];
        newImports.push({
            path: '',
            importType: FileImportType.BY_PATH
        });
        setSelectedFiles(newImports);
    };
    const setImportedPath = async (index, path) => {
        // Update the path of the submitted input field
        const newImports = [...selectedFiles];
        newImports[index].path = path;
        // Set the state
        setSelectedFiles(newImports);
        // Send a message to actually update this
        await sendImportMessage(newImports);
    };
    // Removes a file from being imported
    const removeFileImport = async (index) => {
        const newImports = [...selectedFiles];
        newImports.splice(index, 1);
        setSelectedFiles(newImports);
        // Send a message to actually update this
        await sendImportMessage(newImports);
    };
    useEffect(() => {
        // We load the data after the component renders
        void getDataFiles();
    }, []); // empty array is necessary to have this run only once
    return (React.createElement(DefaultTaskpane, { header: 'Import CSV Files', setCurrOpenTaskpane: props.setCurrOpenTaskpane, taskpaneBody: React.createElement(React.Fragment, null,
            !hasLoadedDataFiles &&
                // We display a loading message while loading the files from the backend
                React.createElement("div", null, "Loading data files in the current folder..."),
            hasLoadedDataFiles &&
                // If we loaded some files, we let users import files
                React.createElement(React.Fragment, null,
                    selectedFiles.map((fileObj, i) => {
                        if (fileObj.importType === FileImportType.CURRENT_FOLDER) {
                            return (React.createElement("div", { key: i, className: 'import-file-select-container' },
                                React.createElement(LargeSelect, { key: i, startingValue: fileObj.path, optionsArray: dataFiles, setValue: (fileName) => { void updateImportedFile(i, fileName, FileImportType.CURRENT_FOLDER); }, extraLarge: true }),
                                React.createElement("div", { className: 'import-file-delete', onClick: () => { void removeFileImport(i); } },
                                    React.createElement(XIcon, null))));
                        }
                        else {
                            return (
                            // See note below about key. 
                            React.createElement("div", { key: i + fileObj.path, className: 'import-file-select-container' },
                                React.createElement(ImportByPathCard
                                /*
                                    Without setting the key like this, when the user deletes a sheet,
                                    it always deletes the last import by path. We're not sure why this
                                    occurs, but making the keys unique in this way fixes it.
                                */
                                , { 
                                    /*
                                        Without setting the key like this, when the user deletes a sheet,
                                        it always deletes the last import by path. We're not sure why this
                                        occurs, but making the keys unique in this way fixes it.
                                    */
                                    key: i + fileObj.path, path: fileObj.path, setImportedPath: (path) => { void setImportedPath(i, path); } }),
                                React.createElement("div", { className: 'import-file-delete', onClick: () => { void removeFileImport(i); } },
                                    React.createElement(XIcon, null))));
                        }
                    }),
                    hasLoadedDataFiles && dataFiles.length === 0 &&
                        /* If there are no files in the Current Folder don't let the use click the Add File from Current Folder Button */
                        React.createElement("div", null,
                            React.createElement("div", { className: 'mb-5' },
                                "There are no files in the current folder to import. ",
                                React.createElement("br", null),
                                " ",
                                React.createElement("br", null),
                                "Upload files to this folder, and then reopen the import taskpane. ",
                                React.createElement("br", null),
                                " ",
                                React.createElement("br", null),
                                "You can read more about importing ",
                                React.createElement("a", { href: 'https://docs.trymito.io/getting-started/importing-data-to-mito', target: "_blank", rel: "noreferrer" },
                                    React.createElement("u", null, "here.")))),
                    hasLoadedDataFiles && dataFiles.length !== 0 &&
                        React.createElement("div", { className: 'blue-button', onClick: addNewCurrentFolderFileImport }, "+ Add File from Current Folder"),
                    props.isLocalDeployment &&
                        React.createElement("div", { className: 'import-add-file-by-path-button', onClick: addNewByPathFileImport },
                            "or ",
                            React.createElement("u", null, "Add File by Path")))) }));
}
export default ImportTaskpane;
//# sourceMappingURL=ImportTaskpane.js.map